@@ -1734,8 +1734,8 @@ def fun_regional_transmission(x, *args):
     omega_t_wind = omega_t[:, n_reg:]
     cf_t_pv = args[0][:, :n_reg]
     cf_t_wind = args[0][:, n_reg:]
-    #  :math:`T_i+D_i(t)-\omega_i \eta_i(t)-\omega_{i + n_reg}
-    #      \eta_{i + n_reg}(t)`
+    #  T_i+D_i(t)-\omega_i \eta_i(t)-\omega_{i + n_reg}
+    #      \eta_{i + n_reg}(t)
     y = (trans_t + dem_t - omega_t_pv * cf_t_pv -
          omega_t_wind * cf_t_wind).flatten()
 
@@ -1962,10 +1962,10 @@ def get_bounds_cstr(lb=None, ub=None, dim=None, sparse=False):
     :type dim: int
     :type sparse: bool
 
-    :returns: A tuple containing matrix :math:`G` and vector
-         :math:`h` defining constraint :math:`G x \le h`.
+    :returns: A tuple containing matrix `G` and vector
+         `h` defining constraint `G x <= h`.
     :rtype: tuple of :py:class:`numpy.array`,
-        or :py:obj:`cvxspmatrix` and :py:obj:`cvxmatrix`.
+        or :py:class:`cvxopt.spmatrix` and :py:class:`cvxopt.cvxmatrix`.
 
     .. note:: Constraint is adapted to the quadprod module.
         To use the CVXOPT module, the oposite should be taken.
@@ -2000,7 +2000,7 @@ def get_bounds_cstr(lb=None, ub=None, dim=None, sparse=False):
             G.append(np.eye(dim))
         h.append(uba)
 
-    # Matrix G and vector h for  :math:`G x \le h`
+    # Matrix G and vector h for `G x <= h`
     h = np.concatenate(h)
     if sparse:
         v = np.concatenate(v)
@@ -2023,9 +2023,9 @@ def get_weighted_sum_cstr(weights, target_sum):
     :type weights: :py:class:`numpy.array`
     :type target_sum: float
 
-    :returns: A tuple containing matrix :math:`A` and vector
-         :math:`b` defining constraint :math:`A x = b`.
-    :rtype: tuple of :py:class:`numpy.array`.
+    :returns: A tuple containing matrix `A` and vector
+         `b` defining constraint `A x = b`.
+    :rtype: tuple of :py:class:`numpy.ndarray`.
     """
     return (np.expand_dims(weights, 0), np.array([target_sum]))
 
@@ -2103,17 +2103,12 @@ def distribute_cv_square(power_mismatch, r):
             all nodes but the first one to be positive.
             Without this bound, one would have:
 
-                * For :math:`r = 0`, the conventional production is uniformly
+                * For `r = 0`, the conventional production is uniformly
                     distributed.
-                * For :math:`r = 1`, the conventional production compensates
+                * For `r = 1`, the conventional production compensates
                     the mismatch between the RES produciton and the demand,
                     i.e. there is no transmission.
-                * For :math:`0 \le r \le 1`, the conventional production is
-                  given by
-
-            .. math::
-                (1 - r) \sum_{i = 1} ^ N \frac{P ^ {\mathrm{_res}}_i - dem_i}{N}
-                - r(P ^ {\mathrm{_res}}_i - dem_i)
+                * For `0 <= r \<= 1`, the conventional production by a mix of the two.
 
         * Since may(for each time step) small
             (the size given by the number of regions)
@@ -2267,13 +2262,7 @@ def distribute_cv_mean_square(power_mismatch, r, window_length=None):
 
 
 def quad_prog_active_set(G, c, A, b=None):
-    """Brute-force active set method to solve convex quadratic program
-
-    .. math::
-
-         \min_x x^t G x + c^t x
-
-         \mathrm{subject~to~} A^t x >= b.
+    """Brute-force active set method to solve convex quadratic program.
 
     :param G: Positive definite matrix.
     :param c: Linear vector.
@@ -2289,7 +2278,7 @@ def quad_prog_active_set(G, c, A, b=None):
         * Optimization problem solution.
         * Lagrange multipliers.
 
-    :rtype: tuple of :py:class:`~numpy.array`
+    :rtype: tuple of :py:class:`numpy.ndarray`
 
     .. note:: This function is used for pedagogical purposes only.
       Favor using more advanced solvers (e.g. quadprog or from scipy).
