@@ -1,13 +1,15 @@
 """Solar-radiation calculations."""
 import numpy as np
+import pandas as pd
+import typing as tp
 import xarray as xr
-import xarray.ufuncs as xrf
-from .solar_calendar import SolarCalendarComputer, _compute_if_none
-from .tracking import SolarTracker
+from e4clim.utils.solar_calendar import (
+    SolarCalendarComputer, compute_if_none)
+from e4clim.utils.tracking import SolarTracker
 
 
 #: Long names for horizontal irradiances.
-LONG_NAMES_HORIZONTAL = {
+LONG_NAMES_HORIZONTAL: tp.Final[tp.Mapping[str, str]] = {
     'global_normal_et': 'Global Normal Extraterrestrial Irradiance',
     'global_horizontal_et': 'Global Horizontal Extraterrestrial Irradiance',
     'global_horizontal_surf': 'Global Horizontal Surface Irradiance',
@@ -21,7 +23,7 @@ LONG_NAMES_HORIZONTAL = {
 }
 
 #: Long names for tilted irradiances.
-LONG_NAMES_TILTED = {
+LONG_NAMES_TILTED: tp.Final[tp.Mapping[str, str]] = {
     'global_tilted_surf': 'Global Tilted Surface Irradiance',
     'direct_trans_factor': 'Direct Transposition Factor',
     'direct_tilted_surf': 'Direct Tilted Surface Irradiance',
@@ -33,7 +35,7 @@ LONG_NAMES_TILTED = {
 }
 
 #: Units of irradiances.
-UNITS = {
+UNITS: tp.Final[tp.Mapping[str, str]] = {
     'global_normal_et': 'W/m2',
     'global_horizontal_et': 'power',
     'clearness_index': '',
@@ -62,12 +64,37 @@ class HorizontalRadiationComputer():
     .. warning:: All computations are for an hourly frequency.
     """
 
+    #: Clearness index.
+    _clearness_index: tp.Optional[xr.DataArray]
+    #: Diffused-radiation model.
+    diffused_model: str
+    #: Elevation threshold.
+    elevation_cut: float
+    #: Global horizontal extraterrestrial radiation.
+    _global_horizontal_et: tp.Optional[xr.DataArray]
+    #: Global horizontal surface radiation.
+    _global_horizontal_surf: tp.Optional[xr.DataArray]
+    #: Latitude.
+    lat: float
+    #: Longitude.
+    lon: float
+    #: Power units.
+    _power_units: str
+    #: Solar-calendar computer.
+    sc: SolarCalendarComputer
+    #: Solar constant.
+    solar_constant: float
+    #: Time index.
+    time: pd.DatetimeIndex
+
     def __init__(
-            self, time=None, lat=None, lon=None,
-            global_horizontal_surf=None, global_horizontal_et=None,
-            clearness_index=None, solar_constant=1367., elevation_cut=None,
-            diffused_model='reindl', solar_computer=None,
-            angles_in_degrees=False, **kwargs):
+            self, time: pd.DatetimeIndex = None, lat: float = None,
+            lon: float = None, global_horizontal_surf: xr.DataArray = None,
+            global_horizontal_et: xr.DataArray = None,
+            clearness_index: float = None, solar_constant: float = 1367.,
+            elevation_cut: float = None, diffused_model: str = 'reindl',
+            solar_computer: SolarCalendarComputer = None,
+            angles_in_degrees: bool = False, **kwargs) -> None:
         """Radiation computations. All computations eventually rely on
         (i) the global horizontal surface radiation,
         :py:obj:`global_horizontal_surf`,
@@ -78,64 +105,54 @@ class HorizontalRadiationComputer():
         coordinates. Thus, at least the global horizontal surface radiation
         or the clearness index should be provided.
 
-        :param time: Time. Default is `None`, in which case
+        :param time: Time. If `None`, :py:obj:`solar_computer` should be given.
+        :param lat: Latitude. If `None`,
           :py:obj:`solar_computer` should be given.
-        :param lat: Latitude. Default is `None`, in which case
-          :py:obj:`solar_computer` should be given.
-        :param lon: Longitude. Default is `None`, in which case
+        :param lon: Longitude. If `None`,
           :py:obj:`solar_computer` should be given.
         :param global_horizontal_surf: Global horizontal surface irradiance.
-          Default is `None`.
         :param global_horizontal_et: Global horizontal extraterrestrial
-          irradiance. Default is `None`.
+          irradiance.
         :param clearness_index: Clearness index.
-          Default is `None`.
         :param solar_constant: Solar constant (yearly averaged, in W/m2).
-          Default is 1367 (W/m2).
         :param elevation_cut: Elevation threshold below which
           direct radiation is considered zero (see note).
           If `None`, use default value of 10 degrees.
         :param diffused_model: Model used to extract diffuse componant
-          of global horizontal radiation. Only the.actuator by
+          of global horizontal radiation. Only the model by
           Reindl *et al.* (1990) is available at the moment.
-          Default is `'reindl'`.
         :param solar_computer: Solar computer to initialize with, instead
           of constructing one from :py:obj:`time`, :py:obj:`lat` and
           :py:obj:`lon`, to speed-up computations.
         :param angles_in_degrees: Whether angles are given in degrees instead
           of radians and should be returned in degrees by
           :py:meth:`.solar_calendar.SolarCalendarComputer.get_angle`.
-        :type time: (sequence of) datetime.
-        :type lat: (sequence of) :py:class:`float`.
-        :type lon: (sequence of) :py:class:`float`.
-        :type global_horizontal_surf: (sequence of) :py:class:`float`.
-        :type global_horizontal_et: (sequence of) :py:class:`float`.
-        :type clearness_index: (sequence of) :py:class:`float`.
-        :type solar_constant: float
-        :type elevation_cut: float
-        :type diffused_model: str
-        :type solar_computer: :py:class:`.solar_calendar.SolarCalendarComputer`
-        :type angles_in_degrees: bool
+
+        :raises AssertionError: if both :py:obj:`global_horizontal_surf` and
+          :py:obj:`clearness_index` are `None`.
         """
         # Make sure that at least the global horizontal surface radiation
         # or the clearness index are provided
-        if (global_horizontal_surf is None) and (clearness_index is None):
-            msg = ('At least the global horizontal surface irradiance '
-                   'or the clearness index should be provided.')
-            raise TypeError(msg)
+        assert (global_horizontal_surf is not None) or (
+            clearness_index is not None), (
+                '"global_horizontal_surf" or "clearness_index" argument'
+                'required')
 
         # Add time, latitude and longitude (in radians) as attributes
         fact = np.deg2rad(1.) if angles_in_degrees else 1.
 
         # Create solar angles computer (all computations in radians)
         if solar_computer is None:
-            self.time, self.lat, self.lon = time, lat * fact, lon * fact
+            self.time = time
+            self.lat = lat * fact
+            self.lon = lon * fact
             self.sc = SolarCalendarComputer(
                 self.time, self.lat, self.lon, angles_in_degrees=False)
         else:
             self.sc = solar_computer
-            self.time, self.lat, self.lon = (
-                self.sc.time, self.sc.lat, self.sc.lon)
+            self.time = self.sc.time
+            self.lat = self.sc.lat
+            self.lon = self.sc.lon
 
         # Initialize all attribute variables to be computed
         self.clear(LONG_NAMES_HORIZONTAL)
@@ -154,7 +171,7 @@ class HorizontalRadiationComputer():
                               else elevation_cut * fact)
         self.diffused_model = diffused_model
 
-    def clear(self, names):
+    def clear(self, names: tp.Iterable[str]) -> None:
         """Set all computed attribute variables to `None`.
 
         :param names: Names of variables to set.
@@ -163,14 +180,13 @@ class HorizontalRadiationComputer():
         for variable_name in names:
             setattr(self, '_' + variable_name, None)
 
-    def _get_power_units(self):
+    def _get_power_units(self) -> str:
         """Try to get power units from
         :py:attr:`self._global_horizontal_surf` or
         :py:attr:`self._global_horizontal_et`, otherwise
         return `'power'`.
 
         :returns: Power units.
-        :rtype: str
         """
         try:
             return self._global_horizontal_surf.attr['units']
@@ -180,11 +196,10 @@ class HorizontalRadiationComputer():
             except (AttributeError, KeyError):
                 return 'power'
 
-    def _add_attributes(self, variable_name):
+    def _add_attributes(self, variable_name: str) -> None:
         """Add `long_name` and `units` attributes to data array.
 
         :param variable_name: Variable name.
-        :type variable_name: str
         """
         # Get variable from variable name
         var = getattr(self, variable_name)
@@ -201,13 +216,12 @@ class HorizontalRadiationComputer():
             pass
 
     @property
-    @_compute_if_none
-    def clearness_index(self):
+    @compute_if_none
+    def clearness_index(self) -> xr.DataArray:
         """Compute the clearness index as the ratio between the
            global (surface) and extraterrestrial horizontal radiations.
 
         :returns: Clearness index.
-        :rtype: (sequence of) :py:class:`float`
         """
         # Compute the clearness index as the ratio between the surface and top
         # of the atmosphere downwelling shortwave radiation.
@@ -223,75 +237,70 @@ class HorizontalRadiationComputer():
         return var
 
     @property
-    @_compute_if_none
-    def global_normal_et(self):
+    @compute_if_none
+    def global_normal_et(self) -> xr.DataArray:
         """Get extraterrestrial solar radiation for a given day of the year,
            taking into account variations or the earth-sun distance.
 
         :returns: Extraterrestrial solar radiation (W/m2).
-        :rtype: (sequence) of float
 
         .. note::
-          * The Fourier series.actuator of Spencer (1971) is used.
+          * The Fourier series model of Spencer (1971) is used.
           * Hourly variations are very small and thus neglected.
         """
         doy_rad = (self.time.dayofyear - 1) * 2*np.pi / 365
 
         return self.solar_constant * (
             1.000110 + 0.034221 *
-            xrf.cos(doy_rad) + 0.001280 * xrf.sin(doy_rad)
-            + 0.000719 * xrf.cos(2*doy_rad) + 0.000077 * xrf.sin(2*doy_rad))
+            np.cos(doy_rad) + 0.001280 * np.sin(doy_rad)
+            + 0.000719 * np.cos(2*doy_rad) + 0.000077 * np.sin(2*doy_rad))
 
     @property
-    @_compute_if_none
-    def global_horizontal_et(self):
+    @compute_if_none
+    def global_horizontal_et(self) -> xr.DataArray:
         """ Get the extraterrestrial horizontal radiation
             from the extraterrestrial radiation and zenith.
 
         :returns: Extraterrestrial horizontal radiation.
-        :rtype: (sequence of) :py:class:`float`
 
         .. note::
           Negative values (between sunrise and sunset) are clipped to zero.
         """
         # Get the extraterrestrial horizontal radiation
-        var = self.global_normal_et * xrf.cos(self.sc.zenith)
+        var = self.global_normal_et * np.cos(self.sc.zenith)
 
         # Keep positive values (between sunrise and sunset)
         return xr.where(var >= 0., var, 0.)
 
     @property
-    @_compute_if_none
-    def global_horizontal_surf(self):
+    @compute_if_none
+    def global_horizontal_surf(self) -> xr.DataArray:
         """ Compute the global horizontal surface radiation as the product
             of the clearness index with the global extraterrestrial
             horizontal radiation.
 
         :returns: Global horizontal surface radiation.
-        :rtype: (sequence of) :py:class:`float`
         """
         # Get the surface horizontal radiation
         return self.clearness_index * self.global_horizontal_et
 
     @property
-    @_compute_if_none
-    def direct_normal_surf(self):
+    @compute_if_none
+    def direct_normal_surf(self) -> xr.DataArray:
         """Get direct normal surface radiation.
 
         :returns: Direct normal radiation.
-        :rtype: (sequence of) :py:class:`float`
         """
         return xr.where(self.direct_horizontal_surf > 1.e-10,
-                        self.direct_horizontal_surf / xrf.cos(self.sc.zenith),
+                        self.direct_horizontal_surf / np.cos(self.sc.zenith),
                         0.)
 
     @property
-    @_compute_if_none
-    def direct_horizontal_surf(self):
+    @compute_if_none
+    def direct_horizontal_surf(self) -> xr.DataArray:
         """ Get the direct horizontal surface radiation.
 
         :returns: Direct horizontal radiation.
-        :rtype: (sequence of) :py:class:`float`
 
         .. note:: The threshold :py:obj:`elevation_cut` should be larger than
           a few degrees in order to prevent numerical overflow in the direct
@@ -310,22 +319,20 @@ class HorizontalRadiationComputer():
         return var
 
     @property
-    @_compute_if_none
-    def diffused_horizontal_surf(self):
+    @compute_if_none
+    def diffused_horizontal_surf(self) -> xr.DataArray:
         """ Get diffuse component of global horizontal surface radiation.
 
         :returns: Diffusive part of the horizontal radiation.
-        :rtype: (sequence of) :py:class:`float`
         """
         return self.global_horizontal_surf * self.diffused_ratio
 
     @property
-    @_compute_if_none
-    def diffused_ratio(self):
+    @compute_if_none
+    def diffused_ratio(self) -> xr.DataArray:
         """Get ratio of diffuse over global horizontal surface radiation.
 
         :returns: Diffuse over global horizontal surface radiation ratio.
-        :rtype: (sequence of) :py:class:`float`
         """
         if self.diffused_model == 'reindl':
             var = self.diffused_ratio_reindl
@@ -333,22 +340,21 @@ class HorizontalRadiationComputer():
         return var
 
     @property
-    @_compute_if_none
-    def diffused_ratio_reindl(self):
+    @compute_if_none
+    def diffused_ratio_reindl(self) -> xr.DataArray:
         """Get ratio of diffuse over global horizontal surface radiation
-        using the.actuator by Reindl * et al.* (1990).
+        using the model by Reindl * et al.* (1990).
 
         :returns: Diffuse over global horizontal surface radiation ratio.
-        :rtype: (sequence of) :py:class:`float`
         """
         var = (1.4 - 1.749 * self.clearness_index
-               + 0.177 * xrf.sin(self.sc.elevation))
+               + 0.177 * np.sin(self.sc.elevation))
         var = xr.where(
             self.clearness_index < 0.3, 1.02 - 0.254 * self.clearness_index
-            + 0.0123 * xrf.sin(self.sc.elevation), var)
+            + 0.0123 * np.sin(self.sc.elevation), var)
         var = xr.where(self.clearness_index > 0.78,
                        0.486 * self.clearness_index
-                       - 0.182 * xrf.sin(self.sc.elevation), var)
+                       - 0.182 * np.sin(self.sc.elevation), var)
         var = xr.where(var < 0.1, 0.1, var)
         var = xr.where(var > 0.97, 0.97, var)
 
@@ -369,13 +375,22 @@ class TiltedRadiationComputer(HorizontalRadiationComputer):
          Solar Radiation on Inclined Surfaces: Models Re-Visited.
          *Energies* 10, 1-28.
     """
+    #: Albedo.
+    albedo: float
+
+    #: Solar tracker.
+    tracker: SolarTracker
 
     def __init__(
-            self, time=None, lat=None, lon=None, global_horizontal_surf=None,
-            global_horizontal_et=None, clearness_index=None,
-            solar_constant=1367., elevation_cut=0., diffused_model='reindl',
-            solar_computer=None, tilt=None, azimuth=None, albedo=0.2,
-            angles_in_degrees=False, **kwargs):
+            self, time: pd.DatetimeIndex = None, lat: float = None,
+            lon: float = None, global_horizontal_surf: xr.DataArray = None,
+            global_horizontal_et: xr.DataArray = None,
+            clearness_index: xr.DataArray = None,
+            solar_constant: float = 1367., elevation_cut: float = None,
+            diffused_model: str = 'reindl',
+            solar_computer: SolarCalendarComputer = None,
+            tilt: float = None, azimuth: float = None, albedo: float = 0.2,
+            angles_in_degrees: bool = False, **kwargs) -> None:
         """Radiation computations. All computations eventually rely on
         (i) the global horizontal surface radiation,
         :py:obj:`global_horizontal_surf`,
@@ -386,61 +401,40 @@ class TiltedRadiationComputer(HorizontalRadiationComputer):
         coordinates. Thus, at least the global horizontal surface radiation
         or the clearness index should be provided.
 
-        :param time: Time. Default is `None`, in which case
+        :param time: Time. If `None`, :py:obj:`solar_computer` should be given.
+        :param lat: Latitude. If `None`,
           :py:obj:`solar_computer` should be given.
-        :param lat: Latitude. Default is `None`, in which case
-          :py:obj:`solar_computer` should be given.
-        :param lon: Longitude. Default is `None`, in which case
+        :param lon: Longitude. If `None`,
           :py:obj:`solar_computer` should be given.
         :param global_horizontal_surf: Global horizontal surface irradiance.
-          Default is `None`.
         :param global_horizontal_et: Global horizontal extraterrestrial
-          irradiance. Default is `None`.
+          irradiance.
         :param clearness_index: Clearness index.
-          Default is `None`.
         :param solar_constant: Solar constant (yearly averaged, in W/m2).
-          Default is 1367 (W/m2).
         :param elevation_cut: Elevation threshold below which
           direct radiation is considered zero (see note).
           If `None`, use default value of 10 degrees.
         :param diffused_model: Model used to extract diffuse componant
-          of global horizontal radiation. Only the.actuator by
+          of global horizontal radiation. Only the model by
           Reindl *et al.* (1990) is available at the moment.
-          Default is `'reindl'`.
         :param tilt: Surface tilt. For east-west axis, north-south axis,
           vertical axis, or two axes, respectively set to `'ew_axis'`,
           `'ns_axis'`, `'vertical'`, or `'two'`.
-          For a tilt equal to the latitude, set to `'lat'`. Default is `None`,
-          in which case computer may not be used to compute tilted irradiances.
-        :param azimuth: Surface azimuth. Default is `None`,
-          in which case computer may not be used to compute tilted irradiances.
+          For a tilt equal to the latitude, set to `'lat'`. If `None`,
+          computer may not be used to compute tilted irradiances.
+        :param azimuth: Surface azimuth. If `None`,
+          computer may not be used to compute tilted irradiances.
         :param solar_constant: Solar constant (yearly averaged, in W/m2).
-          Default is 1367 (W/m2).
         :param albedo: Composite ground albedo used to compute
-          the reflected radiation. Default is 0.2.
+          the reflected radiation.
         :param diffused_model: Model used to extract diffuse componant
-          of global horizontal radiation. Only the.actuator by
+          of global horizontal radiation. Only the model by
           Reindl *et al.* (1990) is available at the moment.
-          Default is `'reindl'`.
         :param solar_computer: Solar computer to initialize with, instead
           of constructing one from :py:obj:`time`, :py:obj:`lat` and
           :py:obj:`lon`, to speed-up computations.
         :param angles_in_degrees: Whether angles are given in degrees instead
           of radians.
-        :type time: (sequence of) datetime.
-        :type lat: (sequence of) :py:class:`float`.
-        :type lon: (sequence of) :py:class:`float`.
-        :type global_horizontal_surf: (sequence of) :py:class:`float`.
-        :type global_horizontal_et: (sequence of) :py:class:`float`.
-        :type clearness_index: (sequence of) :py:class:`float`.
-        :type solar_constant: float
-        :type elevation_cut: float
-        :type diffused_model: str
-        :type solar_computer: :py:class:`.solar_calendar.SolarCalendarComputer`
-        :type tilt: (sequence of) :py:class:`float`, :py:class:`str`
-        :type azimuth: (sequence of) :py:class:`float`, :py:class:`str`
-        :type albedo: (sequence of) :py:class:`float`
-        :type angles_in_degrees: bool
         """
         # Initialize as HorizontalRadiationComputer
         super(TiltedRadiationComputer, self).__init__(
@@ -463,15 +457,14 @@ class TiltedRadiationComputer(HorizontalRadiationComputer):
         self.albedo = albedo
 
     @property
-    @_compute_if_none
-    def global_tilted_surf(self):
+    @compute_if_none
+    def global_tilted_surf(self) -> xr.DataArray:
         """Compute global tilted surface radiation.
 
         :returns: Global tilted surface radiation.
-        :rtype: (sequence of) :py:class:`float`
 
         .. note::
-          * For a general review of the various existing.actuators to calculate
+          * For a general review of the various existing models to calculate
             the global tilted radiation, see Maleki *et al.* (2017)
             and Gueymard (2009).
           * For a general treatment of the available solar radiation,
@@ -482,12 +475,11 @@ class TiltedRadiationComputer(HorizontalRadiationComputer):
                 + self.reflected_tilted_surf)
 
     @property
-    @_compute_if_none
-    def direct_trans_factor(self):
+    @compute_if_none
+    def direct_trans_factor(self) -> xr.DataArray:
         """Get direct transposition factor.
 
         :returns: Direct transposition factor.
-        :rtype: (sequence of) :py:class:`float`
 
         .. seealso::
           :py:meth:`HorizontalRadiationComputer.direct_horizontal_surf`
@@ -500,29 +492,27 @@ class TiltedRadiationComputer(HorizontalRadiationComputer):
                    < np.pi/2))
 
         return xr.where(
-            test, self.tracker.cos_aoi / xrf.cos(self.sc.zenith), 0.)
+            test, self.tracker.cos_aoi / np.cos(self.sc.zenith), 0.)
 
     @property
-    @_compute_if_none
-    def direct_tilted_surf(self):
+    @compute_if_none
+    def direct_tilted_surf(self) -> xr.DataArray:
         """Get direct tilted surface radiation.
 
         :returns: Direct tilted surface radiation.
-        :rtype: (sequence of) :py:class:`float`
         """
         return self.direct_horizontal_surf * self.direct_trans_factor
 
     @property
-    @_compute_if_none
-    def diffused_trans_factor(self):
+    @compute_if_none
+    def diffused_trans_factor(self) -> xr.DataArray:
         """Get diffused transposition factor.
 
         :returns: Diffused transposition factor.
-        :rtype: (sequence of) :py:class:`float`
 
         .. note::
           The diffuse transposition factor is calculated using the
-          HDKR.actuator.
+          HDKR model.
           See Reindl * et al.* (1990) and Maleki * et al.* (2017).
         """
         aniso_idx = xr.where(
@@ -534,56 +524,52 @@ class TiltedRadiationComputer(HorizontalRadiationComputer):
         # Modulating function mod_fun
         mod_fun = xr.where(
             self.global_horizontal_surf > 0.,
-            xrf.sqrt(self.direct_horizontal_surf /
-                     self.global_horizontal_surf),
+            np.sqrt(self.direct_horizontal_surf /
+                    self.global_horizontal_surf),
             0.)
 
         # Isotropic transposition factor
-        transpo_iso = (1. + xrf.cos(self.tracker.surface_tilt)) / 2
+        transpo_iso = (1. + np.cos(self.tracker.surface_tilt)) / 2
 
         # Diffuse transposition factor. See note.
         return (
             aniso_idx * self.direct_trans_factor +
             (1. - aniso_idx) * transpo_iso
-            * (1 + mod_fun * xrf.sin(self.tracker.surface_tilt / 2)**3))
+            * (1 + mod_fun * np.sin(self.tracker.surface_tilt / 2)**3))
 
     @property
-    @_compute_if_none
-    def diffused_tilted_surf(self):
+    @compute_if_none
+    def diffused_tilted_surf(self) -> xr.DataArray:
         """ Get diffuse tilted surface radiation.
 
         :returns: Diffuse tilted surface radiation.
-        :rtype: (sequence of) :py:class:`float`
         """
         return self.diffused_horizontal_surf * self.diffused_trans_factor
 
     @property
-    @_compute_if_none
-    def view_factor(self):
+    @compute_if_none
+    def view_factor(self) -> xr.DataArray:
         """Get view factor for reflected radiation.
 
         :returns: View factor.
-        :rtype: (sequence of) :py:class:`float`
         """
-        return (1. - xrf.cos(self.tracker.surface_tilt)) / 2
+        return (1. - np.cos(self.tracker.surface_tilt)) / 2
 
     @property
-    @_compute_if_none
-    def reflected_trans_factor(self):
+    @compute_if_none
+    def reflected_trans_factor(self) -> xr.DataArray:
         """Get reflected transposition factor.
 
         :returns: Reflected transposition factor.
-        :rtype: (sequence of) :py:class:`float`
         """
         return self.albedo * self.view_factor
 
     @property
-    @_compute_if_none
-    def reflected_tilted_surf(self):
+    @compute_if_none
+    def reflected_tilted_surf(self) -> xr.DataArray:
         """ Get reflected tilted surface radiation.
 
         :returns: Reflected tilted radiation.
-        :rtype: (sequence of) :py:class:`float`
 
         .. note: :
            The reflected tilted radiation is calculated for a flat array
